@@ -126,11 +126,10 @@ class Mapper(object):
 
     @staticmethod
     def set_from_external_ids(external_ids, mappings):
-        rest_optional_values = {}
-        for rest_key, ext_id_key in mappings.items():
-            if ext_id_key in external_ids:
-                rest_optional_values[rest_key] = external_ids[ext_id_key]
-        return rest_optional_values
+        return {
+            rest_key: external_ids.get(ext_id_key)
+            for rest_key, ext_id_key in mappings.items()
+        }
 
 
 class NetworkMapper(Mapper):
@@ -194,9 +193,7 @@ class NetworkMapper(Mapper):
         )
         result = {
             NetworkMapper.REST_NETWORK_ID: str(ls.uuid),
-            NetworkMapper.REST_NETWORK_NAME: str(
-                network_name or ls.name
-            ),
+            NetworkMapper.REST_NETWORK_NAME: network_name or ls.name,
             NetworkMapper.REST_TENANT_ID: tenant_id(),
             NetworkMapper.REST_STATUS: NetworkMapper.NETWORK_STATUS_ACTIVE,
             NetworkMapper.REST_PORT_SECURITY_ENABLED: Mapper._str2bool(
@@ -364,7 +361,7 @@ class PortMapper(Mapper):
                 fixed_ips=fixed_ips,
                 binding_host=binding_host,
                 port_security=port_security,
-                security_groups=security_groups or []
+                security_groups=security_groups
             )
 
     @staticmethod
@@ -485,6 +482,11 @@ class PortMapper(Mapper):
                         subnet_id=PortMapper.REST_PORT_SUBNET_ID
                     )
                 )
+            ip_addr = fixed_ips[0][PortMapper.REST_PORT_IP_ADDRESS]
+            if ip_utils.get_ip_version(ip_addr) is None:
+                raise RestDataError(
+                    'Invalid IP address: {}'.format(ip_addr)
+                )
 
 
 class SubnetMapper(Mapper):
@@ -1079,6 +1081,7 @@ class SecurityGroupRuleMapper(Mapper):
     OVN_SEC_GROUP_RULE_IP_PREFIX = 'ovirt_ip_prefix'
     OVN_SEC_GROUP_RULE_SEC_GROUP_ID = 'ovirt_port_group_id'
     OVN_SEC_GROUP_RULE_DESCRIPTION = 'ovirt_rule_description'
+    OVN_SEC_GROUP_RULE_REMOTE_GROUP_ID = 'ovirt_remote_group_id'
 
     _mandatory_add_data = set(
         [REST_SEC_GROUP_RULE_DIRECTION, REST_SEC_GROUP_RULE_SEC_GROUP_ID]
@@ -1096,6 +1099,7 @@ class SecurityGroupRuleMapper(Mapper):
         REST_SEC_GROUP_RULE_PORT_RANGE_MAX: OVN_SEC_GROUP_RULE_MAX_PORT,
         REST_SEC_GROUP_RULE_PORT_RANGE_MIN: OVN_SEC_GROUP_RULE_MIN_PORT,
         REST_SEC_GROUP_RULE_PROTOCOL: OVN_SEC_GROUP_RULE_PROTOCOL,
+        REST_SEC_GROUP_RULE_REMOTE_GROUP: OVN_SEC_GROUP_RULE_REMOTE_GROUP_ID
     }
 
     @staticmethod
@@ -1127,6 +1131,35 @@ class SecurityGroupRuleMapper(Mapper):
             cls._mandatory_add_data,
             cls._optional_add_data
         )
+        if (
+                rest_data.get(
+                    SecurityGroupRuleMapper.REST_SEC_GROUP_RULE_IP_PREFIX
+                ) and
+                rest_data.get(
+                    SecurityGroupRuleMapper.REST_SEC_GROUP_RULE_REMOTE_GROUP
+                )
+        ):
+            raise BadRequestError(
+                'Only remote_ip_prefix or remote_group_id may be provided.'
+            )
+        prefix = rest_data.get(
+            SecurityGroupRuleMapper.REST_SEC_GROUP_RULE_IP_PREFIX
+        )
+        if prefix:
+            addr_or_prefix = IPNetwork(prefix)
+            ether_type = rest_data.get(
+                    SecurityGroupRuleMapper.REST_SEC_GROUP_RULE_ETHERTYPE
+            )
+
+            if ether_type != 'IPv{version}'.format(
+                    version=addr_or_prefix.version
+            ):
+                raise BadRequestError(
+                    'Conflicting value ethertype '
+                    '{ether_type} for CIDR {cidr}'.format(
+                        ether_type=ether_type, cidr=prefix
+                    )
+                )
 
     @staticmethod
     def rest2row(wrapped_self, func, rest_data, entity_id):
@@ -1156,7 +1189,11 @@ class SecurityGroupRuleMapper(Mapper):
         ip_prefix = rest_data.get(
             SecurityGroupRuleMapper.REST_SEC_GROUP_RULE_IP_PREFIX
         )
+        remote_group_name = rest_data.get(
+            SecurityGroupRuleMapper.REST_SEC_GROUP_RULE_REMOTE_GROUP
+        )
         return func(
             wrapped_self, security_group_id, direction, description,
-            ether_type, port_min, port_max, ip_prefix, protocol
+            ether_type, port_min, port_max, ip_prefix, protocol,
+            remote_group_name
         )
